
#include "outputwidget.h"

#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QScrollArea>
#include <QPushButton>
#include <QPlainTextEdit>
#include <QSyntaxHighlighter>
#include <QTextCharFormat>
#include <QRegularExpression>
#include <QScrollBar>
#include <QVector>
#include <QFileInfo>

#include "common/subprocess.h"

class KeywordHighlighter : public QSyntaxHighlighter {
    Q_OBJECT
public:
    explicit KeywordHighlighter(QTextDocument *parent = nullptr)
        : QSyntaxHighlighter(parent)
    {
        // 定义高亮规则
        HighlightRule ruleWarn;
        ruleWarn.pattern = QRegularExpression("\\bwarn\\b");
        ruleWarn.format.setForeground(QColor(255, 165, 0)); // 橙色
        rules.append(ruleWarn);

        HighlightRule ruleWarning;
        ruleWarning.pattern = QRegularExpression("\\bwarning\\b");
        ruleWarning.format.setForeground(QColor(255, 165, 0)); // 橙色
        rules.append(ruleWarning);

        HighlightRule ruleError;
        ruleError.pattern = QRegularExpression("\\berror\\b");
        ruleError.format.setForeground(Qt::red); // 红色
        rules.append(ruleError);
    }

protected:
    void highlightBlock(const QString &text) override
    {
        for (const HighlightRule &rule : qAsConst(rules))
        {
             QRegularExpressionMatchIterator it = rule.pattern.globalMatch(text);
             while (it.hasNext())
             {
                 QRegularExpressionMatch match = it.next();
                 setFormat(match.capturedStart(), match.capturedLength(), rule.format);
             }
        }
    }

private:
    struct HighlightRule {
        QRegularExpression pattern;
        QTextCharFormat format;
    };
    QVector<HighlightRule> rules;
};

class OutputWidgetPrivate : public QObject
{
    Q_OBJECT

public:
    OutputWidgetPrivate(OutputWidget * parent);
    virtual ~OutputWidgetPrivate();

    void setupUi();

private:
    Q_DECLARE_PUBLIC(OutputWidget)
    OutputWidget * q_ptr;

    QVBoxLayout * m_outputLayout;

    // 功能区
    QPushButton * m_btnBack;
    QPushButton * m_btnBackHome;
    QPushButton * m_btnStop;
    QPlainTextEdit * m_output;
    KeywordHighlighter * m_highlighter = nullptr;

    SubProcess m_subProcess;
};

OutputWidget::OutputWidget(QWidget *parent)
    : QWidget(parent),
      d_ptr(new OutputWidgetPrivate(this))
{
    Q_D(OutputWidget);
    d->setupUi();

    connect(d->m_btnStop, &QPushButton::clicked, [=]() {
        d->m_subProcess.stop();
        d->m_btnStop->setEnabled(false);
        d->m_btnBack->setEnabled(true);
        d->m_btnBackHome->setEnabled(true);
    });

    connect(&d->m_subProcess, &SubProcess::output, [=](QString msg) {
        msg.replace("[1;31m", "");
        msg.replace("[1;33m", "");
        msg.replace("[0m", "");
        d->m_output->appendPlainText(msg);
        if (d->m_highlighter == nullptr) {
            d->m_highlighter = new KeywordHighlighter(d->m_output->document());
        }

        // 自动滚动到底部
        QScrollBar *scrollbar = d->m_output->verticalScrollBar();
        scrollbar->setValue(scrollbar->maximum());
    });

    connect(&d->m_subProcess, &SubProcess::finished, [=]() {
        d->m_btnStop->setEnabled(false);
        d->m_btnBack->setEnabled(true);
        d->m_btnBackHome->setEnabled(true);
    });
}

OutputWidget::~OutputWidget()
{
}

void OutputWidget::startBuild(QString yamlPath)
{
    Q_D(OutputWidget);

    d->m_btnBack->setEnabled(false);
    d->m_btnBackHome->setEnabled(false);
    d->m_btnStop->setEnabled(true);
    d->m_output->setPlainText("");

    QFileInfo fileInfo(yamlPath);
    QString workDir = fileInfo.path(); // 获取文件夹路径
    QString yaml = fileInfo.fileName(); // 获取文件名
    d->m_subProcess.setWorkingDirectory(workDir);

    QStringList args;
    args << "build";
    args << "-f";
    args << "-k";
    args << yaml;
    d->m_subProcess.start("kaiming-builder", args);
}

OutputWidgetPrivate::OutputWidgetPrivate(OutputWidget * parent)
    : QObject(parent),
      q_ptr(parent),
      m_subProcess(this)
{}

OutputWidgetPrivate::~OutputWidgetPrivate()
{}

void OutputWidgetPrivate::setupUi()
{
    Q_Q(OutputWidget);

    m_outputLayout = new QVBoxLayout;
    q->setLayout(m_outputLayout);

    // 操作按钮组
    QHBoxLayout * operationLayout = new QHBoxLayout;
    {
        m_btnBack = new QPushButton;
        m_btnBack->setText(tr("Back"));
        connect(m_btnBack, &QPushButton::clicked, q, &OutputWidget::backOutputClicked);

        m_btnStop = new QPushButton;
        m_btnStop->setText(tr("Stop"));
        m_btnBackHome = new QPushButton(tr("Home Page"));
        connect(m_btnBackHome, &QPushButton::clicked, q, &OutputWidget::backHomeOutputClicked);

        operationLayout->addStretch();
        operationLayout->addWidget(m_btnBack);
        operationLayout->addWidget(m_btnStop);
        operationLayout->addWidget(m_btnBackHome);
    }

    // 日志显示区域
    m_output = new QPlainTextEdit;
    {
        m_output->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        m_output->setReadOnly(true);
    }

    m_outputLayout->addLayout(operationLayout);
    m_outputLayout->addWidget(m_output);

    QScrollArea *scrollArea = new QScrollArea;
    scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
    scrollArea->setWidget(q);
}

#include "outputwidget.moc"
#include "moc_outputwidget.cpp"

