#include "selectdependsdialog.h"

#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QPushButton>
#include <QLineEdit>
#include <QLabel>
#include <QTableWidget>
#include <QStringList>
#include <QGroupBox>
#include <QHeaderView>
#include <QSet>
#include <QList>

#include "logic/KMBuilderApi.h"
#include "logic/KMRef.h"

#define INIT_TABLE(tablewidget) \
    tablewidget->setColumnCount(2); \
    tablewidget->setHorizontalHeaderLabels({"id", "version"}); \
    tablewidget->verticalHeader()->setVisible(false); \
    tablewidget->setEditTriggers(QAbstractItemView::NoEditTriggers); \
    tablewidget->horizontalHeader()->setSectionResizeMode(0, QHeaderView::Stretch); \
    tablewidget->horizontalHeader()->setSectionResizeMode(1, QHeaderView::ResizeToContents); \
    tablewidget->horizontalHeader()->setHighlightSections(true); \
    tablewidget->setSelectionMode(QAbstractItemView::ExtendedSelection); \
    tablewidget->setSelectionBehavior(QAbstractItemView::SelectRows); \
    tablewidget->setMinimumHeight(500); \
    tablewidget->setMinimumWidth(600);

class SelectDependsDialogPrivate : public QObject
{
    Q_OBJECT

public:
    SelectDependsDialogPrivate(SelectDependsDialog * parent, const QString &base, const QString &runtime);
    virtual ~SelectDependsDialogPrivate();

    void setupUi();

public slots:
    void onSearchButtonClicked();
    void transferData(QTableWidget * from, QTableWidget * to);

private:
    Q_DECLARE_PUBLIC(SelectDependsDialog)
    SelectDependsDialog * q_ptr;

    QString m_base;
    QString m_runtime;
    QVector<KMRef> m_refs;

    QLineEdit * m_searchLineEdit;
    QTableWidget * m_leftTable;
    QTableWidget * m_selectedTable;
    QPushButton * m_btnConfirm;
    QPushButton * m_btnCancel;

    QVector<SelectedDepend> m_depends;
};

SelectDependsDialog::SelectDependsDialog(QWidget *parent, const QString &base, const QString &runtime)
    : QDialog(parent),
      d_ptr(new SelectDependsDialogPrivate(this, base, runtime))
{
    Q_D(SelectDependsDialog);
    d->setupUi();

    connect(d->m_btnConfirm, &QPushButton::clicked, this, [=]() {
        d->m_depends.clear();

        int count = d->m_selectedTable->rowCount();
        for (int row = 0; row < count; row++) {
            QTableWidgetItem *idItem = d->m_selectedTable->item(row, 0);
            QTableWidgetItem *versionItem = d->m_selectedTable->item(row, 1);

            SelectedDepend depend;
            depend.id = idItem->text();
            depend.version = versionItem->text();
            d->m_depends << depend;
        }

        accept();
    });

    connect(d->m_btnCancel, &QPushButton::clicked, this, &SelectDependsDialog::reject);
}

const QVector<SelectedDepend>& SelectDependsDialog::getSelected()
{
    Q_D(SelectDependsDialog);

    return d->m_depends;
}

SelectDependsDialog::~SelectDependsDialog()
{}

SelectDependsDialogPrivate::SelectDependsDialogPrivate(SelectDependsDialog * parent, const QString &base, const QString &runtime)
    : QObject(parent),
      q_ptr(parent),
      m_base(base),
      m_runtime(runtime)
{}

SelectDependsDialogPrivate::~SelectDependsDialogPrivate()
{}

void SelectDependsDialogPrivate::setupUi()
{
    Q_Q(SelectDependsDialog);

    QVBoxLayout * vLayout = new QVBoxLayout;
    q->setLayout(vLayout);

    //搜索框
    QHBoxLayout * searchLayout = new QHBoxLayout;
    QLabel * idPackage = new QLabel("id");
    m_searchLineEdit = new QLineEdit;
    m_searchLineEdit->setFixedWidth(400);
    QPushButton * searchButton = new QPushButton("🔍");
    searchLayout->addStretch();
    searchLayout->addWidget(idPackage);
    searchLayout->addWidget(m_searchLineEdit);
    searchLayout->addWidget(searchButton);
    searchLayout->addStretch();

    // 左右表格
    QHBoxLayout * tableLayout = new QHBoxLayout;
    {
        QGroupBox * selectGroup = new QGroupBox(tr("Please Select"));
        QVBoxLayout * selectGroupLayout = new QVBoxLayout;
        selectGroup->setLayout(selectGroupLayout);
        m_leftTable = new QTableWidget;
        selectGroupLayout->addWidget(m_leftTable);
        INIT_TABLE(m_leftTable)

        QVBoxLayout * transLayout = new QVBoxLayout;
        QPushButton * right = new QPushButton(" >> ");
        right->setFlat(true);
        right->setFixedSize(36, 36);
        QPushButton * left = new QPushButton(" << ");
        left->setFlat(true);
        left->setFixedSize(36, 36);
        transLayout->addStretch();
        transLayout->addWidget(right);
        transLayout->addSpacing(15);
        transLayout->addWidget(left);
        transLayout->addStretch();

        QGroupBox * selectedGroup = new QGroupBox(tr("Selected"));
        QVBoxLayout * selectedGroupGroupLayout = new QVBoxLayout;
        selectedGroup->setLayout(selectedGroupGroupLayout);
        m_selectedTable = new QTableWidget;
        selectedGroupGroupLayout->addWidget(m_selectedTable);
        INIT_TABLE(m_selectedTable)

        tableLayout->addWidget(selectGroup);
        tableLayout->addLayout(transLayout);
        tableLayout->addWidget(selectedGroup);

        connect(right, &QPushButton::clicked, this, [=]() {
            this->transferData(m_leftTable, m_selectedTable);
        });

        connect(left, &QPushButton::clicked, this, [=]() {
            this->transferData(m_selectedTable, m_leftTable);
        });
    }

    // 确定按钮
    QHBoxLayout * confirmLayout = new QHBoxLayout;
    m_btnConfirm = new QPushButton(tr("Confirm"));
    m_btnCancel = new QPushButton(tr("Cancel"));
    confirmLayout->addStretch();
    confirmLayout->addWidget(m_btnConfirm);
    confirmLayout->addWidget(m_btnCancel);
    confirmLayout->addStretch();

    vLayout->addLayout(searchLayout);
    vLayout->addLayout(tableLayout);
    vLayout->addSpacing(15);
    vLayout->addLayout(confirmLayout);
    vLayout->addSpacing(15);

    m_refs = KMBuilderSearch::search("depend", m_base, m_runtime);

    connect(searchButton, &QPushButton::clicked, this, &SelectDependsDialogPrivate::onSearchButtonClicked);
    connect(m_searchLineEdit, &QLineEdit::textChanged, this, &SelectDependsDialogPrivate::onSearchButtonClicked);
}

void SelectDependsDialogPrivate::onSearchButtonClicked()
{
    //获取过滤条件
    QString id = m_searchLineEdit->text();
    QVector<KMRef> depends = KMBuilderApi::filter(m_refs, id); //过滤

    m_leftTable->clearContents();
    m_leftTable->setRowCount(0);
    int indexOfRow = 0;
    for (auto& depend : depends)
    {
        m_leftTable->insertRow(indexOfRow);

        QTableWidgetItem * id = new QTableWidgetItem;
        id->setText(depend.m_id);
        id->setToolTip(depend.m_id);
        m_leftTable->setItem(indexOfRow, 0, id);

        QTableWidgetItem * version = new QTableWidgetItem;
        version->setText(depend.m_version);
        version->setToolTip(depend.m_version);
        m_leftTable->setItem(indexOfRow, 1, version);

        ++indexOfRow;
    }
}

void SelectDependsDialogPrivate::transferData(QTableWidget * from, QTableWidget * to)
{
    // 获取所有选中的行索引（去重处理）
    QSet<int> selectedRows;
    for (QTableWidgetItem *item : from->selectedItems()) {
        selectedRows.insert(item->row());
    }

    // 倒序处理防止删除行时索引变化
    QList<int> rows = selectedRows.values();
    std::sort(rows.rbegin(), rows.rend());

    foreach (int row, rows) {
        // 获取数据
        QTableWidgetItem *idItem = from->item(row, 0);
        QTableWidgetItem *versionItem = from->item(row, 1);

        // 添加到右侧表格
        int newRow = to->rowCount();
        to->insertRow(newRow);

        QTableWidgetItem * id = new QTableWidgetItem;
        id->setText(idItem->text());
        id->setToolTip(idItem->text());
        to->setItem(newRow, 0, id);

        QTableWidgetItem * version = new QTableWidgetItem;
        version->setText(versionItem->text());
        version->setToolTip(versionItem->text());
        to->setItem(newRow, 1, version);

        // 从左侧表格删除
        from->removeRow(row);
    }
}

#include "selectdependsdialog.moc"
#include "moc_selectdependsdialog.cpp"
